local i, keystone,_,bag,bagSize



function KeystoneRollCall:keystonesMatch(k1,k2)
    if k1 == k2 then return true end
    local o1Type = type(k1)
    local o2Type = type(k2)
    if o1Type ~= o2Type then return false end
    if o1Type ~= 'table' then return false end

    local keySet = {}

    for key1, value1 in pairs(k1) do
        local value2 = k2[key1]
        if value2 == nil or self:keystonesMatch(value1, value2) == false then
			--print(key1,k1[key1],k2[key1])
            return false
        end
        keySet[key1] = true
    end

    for key2, _ in pairs(k2) do
        if not keySet[key2] then
			--print(key2,k1[key2],k2[key2])
			return false 
		end
    end
    return true
end

function KeystoneRollCall:purgeExpiredKeystones()
	local playerKeystone = self:getPlayerKeystone()
	if not playerKeystone or not playerKeystone.affix1 or playerKeystone.affix1 <= 0 then
		return 
	end
	
	for i = #self.db.global.keystones, 1, -1 do
		local affix1 = self.db.global.keystones[i].affix1;
		if affix1 and affix1 > 0 and affix1 ~= playerKeystone.affix1 then
			tremove(self.db.global.keystones,i)
		end
	end
end

function KeystoneRollCall:extractKeystone(k)
	if not k then
		return
	end
	return (k.name or ""), (k.class or ""),(k.faction and 1 or 0),(k.dungeon or 0),(k.level or 0),(k.affix1 or 0),(k.affix2 or 0),(k.affix3 or 0),(k.bestLevel or 0),(k.bestDungeon or 0),(KeystoneRollCall.newestVersionFound or 0),(k.ilvl or 0),(k.equippedIlvl or 0)
end

function KeystoneRollCall:buildKeystone(args,owner)
	local keystone = {}	
	keystone.name = args[1]
	keystone.class = args[2]
	keystone.faction = args[3] == "1"
	keystone.dungeon = tonumber(args[4])
	keystone.level = tonumber(args[5])
	keystone.affix1 = tonumber(args[6])
	keystone.affix2 = tonumber(args[7])
	keystone.affix3 = tonumber(args[8])
	keystone.bestLevel = tonumber(args[9])
	keystone.bestDungeon = tonumber(args[10])
	keystone.version = args[11]
	keystone.ilvl = tonumber(args[12])
	keystone.equippedIlvl = tonumber(args[13])
	keystone.isPlayers = false;
	keystone.owner = owner
	keystone.lastSeen = time()

	local oldKeystoneIndex = self:getKeystoneIndex(keystone.name)
	if oldKeystoneIndex and oldKeystoneIndex > 0 then
		local oldKeystone = self.db.global.keystones[oldKeystoneIndex]
		if oldKeystone.isPlayers then
			keystone.isPlayers = true;
		end
	end

	self:populateCharacterData(keystone)
	return self:saveKeystone(keystone)
end


function KeystoneRollCall:saveKeystone(keystone)
	local oldKeystone = self:getKeystone(keystone.name);
	
	if not keystone then
		return nil, oldKeystone;
	end

	local isPlayer = keystone.name == KeystoneRollCall.playerName ;
	
	if not isPlayer then
		local playerKeystone = self:getPlayerKeystone()
		if playerKeystone  and playerKeystone.affix1 and playerKeystone.affix1 > 0 and keystone.affix1 and keystone.affix1 > 0 and keystone.affix1 ~= playerKeystone.affix1 then
			return nil, oldKeystone;
		end
	end
	
	

	if oldKeystone then
		if not self:keystonesMatch(oldKeystone,keystone) then
			if not isPlayer or (keystone.level and keystone.level >0) or not oldKeystone.level or oldKeystone.level <=0 then
				local oldKeystoneIndex = self:getKeystoneIndex(keystone.name);
				self.db.global.keystones[oldKeystoneIndex] = keystone;
				if isPlayer then	
					self:purgeExpiredKeystones()
				end
				return keystone, oldKeystone
			end
		end
			
		return nil, oldKeystone
	end

	tinsert(self.db.global.keystones,keystone);
	if isPlayer and keystone.affix1 and keystone.affix1 > 0 then	
		self:purgeExpiredKeystones()
	end
	
	return keystone, oldKeystone;
end 

function KeystoneRollCall:buildPlayerKeystone()
	local mapTable = C_ChallengeMode.GetMapTable();
	local bestLevel = 0;
	local bestDungeon;
	for i, dungeonId in pairs(mapTable) do
		local x, weeklyBestTime, weeklyBestLevel = C_ChallengeMode.GetMapPlayerStats(dungeonId);
		if weeklyBestLevel and weeklyBestLevel > bestLevel then
			bestLevel = weeklyBestLevel;
			bestDungeon = dungeonId;
		end
	end
	
	local ilvl, equippedIlvl = GetAverageItemLevel()
	
	local keystone = {}
	keystone.name = KeystoneRollCall.playerName;
	keystone.class = UnitClass("player");
	keystone.isPlayers = true;
	keystone.bestLevel = bestLevel
	keystone.bestDungeon = bestDungeon
	keystone.faction = UnitFactionGroup("player") == "Alliance"
	keystone.dungeon = 0;
	keystone.ilvl = math.floor(ilvl) 
	keystone.equippedIlvl = math.floor(equippedIlvl)
	keystone.level = 0;
	local newKey = false;
	for bag = 0, NUM_BAG_SLOTS do
		bagSize = GetContainerNumSlots(bag);
		if bagSize > 0 then
			for i = 1, bagSize do
				if (GetContainerItemID(bag, i) == 138019) then
					local keyParts = { strsplit(':[', GetContainerItemLink(bag, i)) }
					
					keystone.dungeon = tonumber(keyParts[2]);
					keystone.level = tonumber(keyParts[3]);
					keystone.affix1 = tonumber(keyParts[4]) or 0;
					keystone.affix2 = tonumber(keyParts[5]) or 0;
					keystone.affix3 = tonumber(keyParts[6]) or 0;
					self:populateCharacterData(keystone)
					return self:saveKeystone(keystone)
				end
			end
		end
	end

	self:populateCharacterData(keystone)
	return self:saveKeystone(keystone)
end

function KeystoneRollCall:populateCharacterData(keystone)
	if not keystone or not keystone.name then
		return
	end
	local data = self:getCharacterData(keystone.name)
	if not data.bestRuns then
		data.bestRuns = {}
	end
	if keystone.bestLevel and keystone.bestLevel >0 then
		local runEntry = {level = keystone.bestLevel, dungeon = keystone.bestDungeon, affix1 = keystone.affix1, affix2 = keystone.affix2, affix3 = keystone.affix3}
		
		local newAffix1 = data.bestRuns[1] and (data.bestRuns[1].affix1 and data.bestRuns[1].affix1 >0 and data.bestRuns[1].affix1 ~= runEntry.affix1 );
		local newAffix2 = data.bestRuns[1] and (data.bestRuns[1].affix2 and data.bestRuns[1].affix2 >0 and data.bestRuns[1].affix2 ~= runEntry.affix2 );
		local newAffix3 = data.bestRuns[1] and (data.bestRuns[1].affix3 and data.bestRuns[1].affix3 >0 and data.bestRuns[1].affix3 ~= runEntry.affix3 );
		if data.bestRuns[1] and data.bestRuns[1].affix1 and (newAffix1 or newAffix2 or newAffix3)then
			tinsert(data.bestRuns,1,runEntry)
		else
			data.bestRuns[1] = runEntry
		end
	end

	data.class = keystone.class
	data.faction = keystone.faction
	data.isPlayers = keystone.isPlayers
	data.ilvl = keystone.ilvl
	data.equippedIlvl = keystone.equippedIlvl
	if not data.bestLevel or data.bestLevel < keystone.bestLevel then
		data.bestLevel = keystone.bestLevel
		data.bestDungeon = keystone.bestDungeon
	end
	data.lastSeen = time()
end

function KeystoneRollCall:getKeystoneIndex(characterName)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == characterName then
			return i
		end
	end
	return -1
end

function KeystoneRollCall:getIlvlLine(keystone)
	local ilvlLine = ""

	if keystone.ilvl and tonumber(keystone.ilvl) > 0 then
		local dif = keystone.ilvl - keystone.equippedIlvl
		if dif == 0 then
			ilvlLine = tostring(keystone.equippedIlvl)
		else
			ilvlLine = keystone.equippedIlvl .." +"..(keystone.ilvl - keystone.equippedIlvl)..""
		end
	end
	return ilvlLine
end

function KeystoneRollCall:forEachAlt(main,callable)
	local mainKeystone = self:getKeystone(main)
	for i, keystone in pairs(self.db.global.keystones) do
		if mainKeystone and mainKeystone.owner and keystone.owner == mainKeystone.owner and keystone.name ~= mainKeystone.name then
			callable(keystone,mainKeystone)
		end
	end
end

function KeystoneRollCall:getKeystoneByUnit(unit)

	local name = KeystoneRollCall:getFullName(unit)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == name then
			return keystone
		end
	end
	return nil
end

function KeystoneRollCall:getKeystone(characterName)
	for i, keystone in pairs(self.db.global.keystones) do
		if keystone.name == characterName then
			return keystone
		end
	end
	return nil
end

function KeystoneRollCall:getPlayerKeystoneIndex()
	return self:getKeystoneIndex(KeystoneRollCall.playerName)
end

function KeystoneRollCall:getPlayerKeystone()
	return self:getKeystone(KeystoneRollCall.playerName)
end

function KeystoneRollCall:getFullName(unit)
	local name, realm = UnitFullName(unit)
	realm = realm and realm ~= "" and realm or KeystoneRollCall.playerRealm
	if not realm then
		local playerName, playerRealm = UnitFullName("player")
		KeystoneRollCall.playerName = playerName
		KeystoneRollCall.playerRealm = playerRealm
		realm = playerRealm
	end
	local fullName = name .. "-" .. realm
	return fullName
end

function KeystoneRollCall:getCharacterData(fullName)
	local data = self.db.global.characters[fullName]
	if not data then
		data = {}
		self.db.global.characters[fullName]	=	data
	end
	return data
end

function KeystoneRollCall:getDungeonName(dungeon)
	return dungeon and C_ChallengeMode.GetMapInfo(dungeon) or ""
end

function KeystoneRollCall:getBestRunLine(bestLevel,bestDungeon)
	local bestLevelLine = ""
	if bestLevel and bestLevel > 0 and bestDungeon then
		bestLevelLine = "+" .. KeystoneRollCall:colorLevelMaxReward(bestLevel,bestLevel).." |cFFFFFFFF" .. KeystoneRollCall:getDungeonName(bestDungeon).."|r"
	end
	return bestLevelLine
end

function KeystoneRollCall:showTooltip(fullName,anchorFrame)
	local data = self:getCharacterData(fullName)
	if data and data.characterName and data.characterName~= "" then
		GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
		GameTooltip:ClearAllPoints();
		GameTooltip:SetPoint("TOPRIGHT",anchorFrame,"BOTTOMRIGHT");

		data.characterName = data.characterName or ""
		data.class = data.class or ""
		GameTooltip:SetText(self:colorClass(data.characterName or "",data.class or ""))
		GameTooltip:AddDoubleLine(" ");
		GameTooltip:AddDoubleLine(" Realm",data.realm);
		if data.faction ~= nil then
			GameTooltip:AddDoubleLine(" Faction",KeystoneRollCall:colorFaction(data.faction and "Alliance" or "Horde",data.faction));
		end
		if data.race then
			GameTooltip:AddDoubleLine(" Race",data.race);
		end
		GameTooltip:AddDoubleLine(" Class",self:colorClass(data.class,data.class));
		if data.level then
			GameTooltip:AddDoubleLine(" Level",data.level);
		end
		if data.ilvl and data.equippedIlvl then
			GameTooltip:AddDoubleLine(" Item Level",self:getIlvlLine(data));
		end

		if data.jointRuns and #data.jointRuns > 0 then
			GameTooltip:AddDoubleLine(" Joint Run History");
			for i = 1,5 do
				if i <= #data.jointRuns then 
					GameTooltip:AddLine("      |cFFFFFFFF"..KeystoneRollCall:getBestRunLine(data.jointRuns[i].level,data.jointRuns[i].dungeon).."|r");
				end
			end
		end

		if data.bestRuns and #data.bestRuns > 0 then
			GameTooltip:AddDoubleLine(" Best Run History");
			for i = 1,5 do
				if i <= #data.bestRuns then 
					GameTooltip:AddLine("      |cFFFFFFFF"..KeystoneRollCall:getBestRunLine(data.bestRuns[i].level,data.bestRuns[i].dungeon).."|r");
				end
			end
		end
		local altFound = false;
		if KeystoneRollCall.db.global.guildOnlineShowKnownAlts then
			KeystoneRollCall:forEachAlt(fullName,function(altKeystone,mainsKeystone)
				if altKeystone.name then
					if not altFound then
						GameTooltip:AddDoubleLine(" Known Alts");
						altFound = true;
					end
					local altNameParts =  {strsplit("-",altKeystone.name)};
					GameTooltip:AddLine("      "..self:colorClass(altNameParts[1],altKeystone.class));

				end
			end)
		end


		if data.lastSeen then
			GameTooltip:AddDoubleLine(" ");
			GameTooltip:AddDoubleLine(" ",date("%m/%d/%y",data.lastSeen))
		end

		if not GameTooltip.bg then
			GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
			GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
			GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
			GameTooltip.bg:SetColorTexture(0,0,0,1)

		end
		GameTooltip.bg:Show()
		GameTooltip:Show()
	end
end