local tab = {}
tab.index = 300
tab.text = "Guild (Online)";
tab.value = "guild_online";

tab.columnSort = function (table, rowa, rowb, sortbycol)
	local asc = table.cols[sortbycol].sort == "asc"
	local ka = KeystoneRollCall.tableData[rowa][1]
	local kb = KeystoneRollCall.tableData[rowb][1]
	return KeystoneRollCall:CompareKeystones("name", asc, ka, kb);
end

tab.populate = function()
	local includeKeyless = KeystoneRollCall.db.global.guildOnlineTabShowKeyless
	
	for i = 1, GetNumGuildMembers() do
		local fullName, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile, canSoR, reputation = GetGuildRosterInfo(i)
		
		local nameParts =  {strsplit("-",fullName)};
	
		local data  = KeystoneRollCall:getCharacterData(fullName)
		data.class = class
		data.level = level
		data.name = fullName
		data.characterName = nameParts[1]
		data.realm = nameParts[2]
		data.faction = UnitFactionGroup("player") == "Alliance"
		
		
		if online then
			local keystone = KeystoneRollCall:getKeystone(fullName)
			
			if includeKeyless or keystone and keystone.level and keystone.level > 0 then
			
				if not keystone then
					tinsert(KeystoneRollCall.tableData, {
						{['name'] =fullName,['class'] = class,dungeon = -1,level = -1, dummy = true},
						"",
						KeystoneRollCall:colorClass(nameParts[1],class),
						KeystoneRollCall:colorClass(class,class),
						"",
						"",
						"",
						""
					});
				else
					tinsert(KeystoneRollCall.tableData, {
						keystone,
						"",
						KeystoneRollCall:colorClass(nameParts[1],class),
						KeystoneRollCall:colorClass(class,class),
						KeystoneRollCall:getIlvlLine(keystone),
						keystone.level == 0 and "" or ("+"..KeystoneRollCall:colorLevelDifficulty(keystone.level,keystone.level)),
						C_ChallengeMode.GetMapInfo(keystone.dungeon),
						KeystoneRollCall:getBestRunLine(keystone.bestLevel,keystone.bestDungeon)
					});
					
					if KeystoneRollCall.db.global.guildOnlineShowKnownAlts then
						KeystoneRollCall:forEachAlt(fullName,function(altKeystone,mainsKeystone)
							if altKeystone.name and (includeKeyless or altKeystone and altKeystone.level and altKeystone.level > 0) then
								local altNameParts =  {strsplit("-",altKeystone.name)};
								
								tinsert(KeystoneRollCall.tableData, {
									altKeystone,
									KeystoneRollCall:colorClass(nameParts[1],class),
									KeystoneRollCall:colorClass(altNameParts[1],altKeystone.class),
									KeystoneRollCall:colorClass(altKeystone.class,altKeystone.class),
									KeystoneRollCall:getIlvlLine(altKeystone),
									altKeystone.level == 0 and "" or ("+"..KeystoneRollCall:colorLevelDifficulty(altKeystone.level,altKeystone.level)),
									C_ChallengeMode.GetMapInfo(altKeystone.dungeon),
									KeystoneRollCall:getBestRunLine(altKeystone.bestLevel,altKeystone.bestDungeon)
								});
							end
						end)
					end
				end
			end
		end
	end
	
	KeystoneRollCall.scrollTable:SortData();
end

tab.setup = function(container)
	local cols = {{
			['name'] = '',
			['width'] = 1,
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "name",
			['align'] = 'LEFT'
		},{
			['name'] = 'Alt Of',
			['width'] = 85,
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "owner",
			['align'] = 'LEFT',
		},{
			['name'] = 'Character',
			['width'] = 85,
			['align'] = 'LEFT', 
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "name",
		},{
			['name'] = 'Class',
			['width'] = 90,
			['align'] = 'LEFT', 
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "class",
		},{
			['name'] = 'ILvl',
			['width'] = 50,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "equippedIlvl",
		},{
			['name'] = 'Level',
			['width'] = 40,
			['align'] = 'RIGHT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "level",
		},{
			['name'] = '  Dungeon',
			['width'] = 150,
			['align'] = 'LEFT',
			["comparesort"] = KeystoneRollCall.columnSort,
			['field'] = "dungeon",
		},{
			['name'] = "Best Run (This Week)",
			['field'] = "bestLevel",
			["comparesort"] = KeystoneRollCall.columnSort,
			['width'] = 180,
			['align'] = 'LEFT',
		}
	}
	
	KeystoneRollCall.tableData = {};
	KeystoneRollCall.dummyKeystones = {};
	KeystoneRollCall.scrollTable = KeystoneRollCall.ScrollingTable:CreateST(cols,24);
	KeystoneRollCall.scrollTable:SetData(KeystoneRollCall.tableData, true);

	KeystoneRollCall.scrollTable:RegisterEvents({
		['OnClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if not KeystoneRollCall.tableData or not KeystoneRollCall.tableData[realrow] then
				return
			end
			if KeystoneRollCall.tableData[realrow][1].dummy then
				SendChatMessage("If you get the addon Keystone Roll-Call we can easily view each other's keystones, as well as keystones from across our guilds.","WHISPER",  "Common", KeystoneRollCall.tableData[realrow][1].name);
			elseif KeystoneRollCall.tableData[realrow][1] then
				local keystone = KeystoneRollCall.tableData[realrow][1];
				if btn == "RightButton" then
					if UnitExists("target") and GetUnitName("target", true) then
						SendChatMessage(KeystoneRollCall:keystoneToString(keystone),"WHISPER", "COMMON",  GetUnitName("target", true));
					end
				else
					SendChatMessage(KeystoneRollCall:keystoneToString(keystone),KeystoneRollCall:getGroupChatChannel() ,  "Common")
				end
			end
		end,	
		['OnDoubleClick'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable, btn,...)
			if KeystoneRollCall.tableData and KeystoneRollCall.tableData[realrow] and not KeystoneRollCall.tableData[realrow][1].dummy then
				SendChatMessage(KeystoneRollCall:keystoneToString(KeystoneRollCall.tableData[realrow][1]),"GUILD" ,  "Common")
			end
		end,	
		['OnEnter'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			if not realrow or not KeystoneRollCall.tableData[realrow] then
				return
			end
			
			
			local key = KeystoneRollCall.tableData[realrow][1]
			local data  = KeystoneRollCall:getCharacterData(key.name)
			--print(key.dummy, data)
			
			
			if key.dummy then
				KeystoneRollCall.frame:SetStatusText("Click: Suggest Addon")
			else
				KeystoneRollCall.frame:SetStatusText("Left Click: Send to Party   -   Double Click: Send to Guild   -   Right Click: Send to Target")
			end
			
			
			KeystoneRollCall:showTooltip(key.name,rowFrame)
		end,
		['OnLeave'] = function(rowFrame, cellFrame, data, cols, row, realrow, column, scrollingTable,...)
			KeystoneRollCall.frame:SetStatusText("")
			if GameTooltip.bg then
				GameTooltip.bg:Hide()
			end
			GameTooltip:Hide()
		end,
	});
	
	local tableWrapper = KeystoneRollCall.AceGUI:Create('lib-st'):WrapST(KeystoneRollCall.scrollTable);
	tableWrapper.head_offset = 15;
	container:AddChild(tableWrapper);
end

KeystoneRollCall.tabs[tab.value] = tab;